      subroutine atmosinit(atname)
      
      implicit none
      
      integer nHnu,nHnumax
      parameter(nHnumax=25000)
      real atmosnu,Hnuint
      dimension atmosnu(nHnumax),Hnuint(nHnumax)
      common /atmos_interp/atmosnu,Hnuint,nHnu


      CHARACTER atname*50

c-----------------------------------------------------------------------
c      write(*,*) 'Filename with the model atmosphere spectrum'
c      read(*,'(a50)') name
c      name='../parfiles/flx4000kg35.par'
      call partintgr(atname,atmosnu,Hnuint,nHnu)
      
      return
      end

c **********************************************************************
        SUBROUTINE partintgr(name,w,p,n)
c ----------------------------------------------------------------------
c   Reads in a two-column array containing wavelengths and fluxes
c   Calculates partial integrals at every wavelength point
c   Normalizes the integrals on the bolometric flux
c   Uses real function tr2 for integration
c-----------------------------------------------------------------------
c                                [Anatoly Miroshnichenko, Mar. 1998]
c ======================================================================       
        implicit none

        include 'dust.com'
      
        integer nHnu,nHnumax
        parameter(nHnumax=25000)

        CHARACTER name*50
        CHARACTER*80 header1,header2,header3
        INTEGER i,n
        REAL w(nHnumax),winv(nHnumax),f(nHnumax),chif(nHnumax),
     +       p(nHnumax),fbol,tr2
c-----------------------------------------------------------------------
        open(1,file=name,status='old')
        read(1,*) header1
        read(1,*) header2
        read(1,*) header3
        i=1
 1      read(1,*,end=2) w(i),winv(i),f(i)

	    w(i)=1.24*winv(i)
            call opacset(w(i))
            chif(i)=kappa*f(i)

            if(i.eq.1) then
                p(i)=0.
            else
	        p(i)=tr2(f,winv,i)
            endif

            i=i+1

        goto 1

 2      close(1)
        
	n=i-1
        
	fbol=p(n)
        do i=1,n
            p(i)=p(i)/fbol
        end do

	kappaf=tr2(chif,winv,i)/fbol

        write(*,*) 'kappaf=',kappaf        
	
	return
        end

c **********************************************************************
        REAL FUNCTION tr2(a,b,n)
c ----------------------------------------------------------------------
c   Numerical integration using the trapezium rule
c-----------------------------------------------------------------------
c                                [Anatoly Miroshnichenko, Feb. 1998]
c ======================================================================
        implicit none
        INTEGER i,n
        REAL a(n),b(n)
c-----------------------------------------------------------------------
        tr2=0.
        do 1 i=2,n
1       tr2=tr2+(a(i)+a(i-1))*(b(i-1)-b(i))/2.
        return
        end
c **********************************************************************
      
      
      real function atmosfreq()

      implicit none

      integer i0,i1,i
      real xi,f,P0,P1,P

      integer nHnu,nHnumax
      parameter(nHnumax=25000)
      real atmosnu,Hnuint
      dimension atmosnu(nHnumax),Hnuint(nHnumax)
      common /atmos_interp/atmosnu,Hnuint,nHnu

      real ran2
      integer iseed,idum
      common /ranparm/ iseed,idum

      
      xi=ran2(idum)

      i0=1
      i1=nHnu

      P0=  Hnuint(i0)
      if(xi.lt.P0) then
          atmosfreq=atmosnu(1)
	  return
      endif

      P1=  Hnuint(i1)
      if (P1 .gt. 1.) then
          write(*,*) 'atmosfreq: P1>1, P1 = ',P1
	  P1=1.
      endif
      if (xi.ge.P1) then
          atmosfreq=atmosnu(nHnu)
          return
      endif

      do while((i1-i0).gt.1)

          i=(i0+i1)/2
          P=Hnuint(i)

	  if (P .gt. 1.) then
              write(*,*)'atmosfreq: P>1; i,P = ',i,P
	      P=1.
	  endif

          if (P .lt. P0) then
	      write(*,*) 'atmosfreq: P<P0; P0,P,P1 = ',P0,P,P1
	      P0=P
	      i0=i
	  endif
          if (P .gt. P1) then
	      write(*,*) 'atmosfreq: P>P1; P0,P,P1 = ',P0,P,P1
	      P1=P
	      i1=1
	  endif
	  
          if (P.gt.xi) then
              i1=i
              P1=P
          else
              i0=i
              P0=P
          endif

      end do

      if (P1.le.P0) then
          write(*,*) 'atmosfreq: P1<=P0; P0, P1 = ',P0,P1
	  stop
      endif
      
c      f=(xi-P0)/(P1-P0)
      if (xi.lt.P0) then
          write(*,*) 'atmosfreq: xi<P0, xi,P0 = ',xi,P0
          f=0.
      elseif (xi.gt.P1) then
          write(*,*) 'atmosfreq: xi>P1, xi,P1 = ',xi,P1
          f=1.
      elseif (P0.le.0.) then
          write(*,*) 'atmosfreq: P0<=0, P0,i0,i1 = ',P0,i0,i1
	  f=0.5
      elseif (P1.le.0.) then
          write(*,*) 'atmosfreq: P1<=0, P1 = ',P1
	  stop
      elseif (xi.le.0.) then
          write(*,*) 'atmosfreq: xi<=0, xi = ',xi
	  stop
      else
          f=log(xi/P0)/log(P1/P0)
      endif
      if ((f.lt.0.).or.(f.ge.1.)) then
          write(*,*) 'atmosfreq: f = ',f,' out of bounds'
      endif

      atmosfreq= atmosnu(i0) * (atmosnu(i1)/atmosnu(i0))**f

      return
      end

      real function bbfreq(Teff)
      
      implicit none

      include 'random.com'
      
      integer i      
      real*8 Teff
      real z0,z1,z2,z3,z4
      logical first
      
      real ran2

      integer izmax
      parameter(izmax=145)
      real inczeta
      dimension inczeta(izmax)
      common /izeta/ inczeta

      save first
      data first /.true./
      
      if (first) then
          call zetainit()
	  write(*,*) 'initializing inczeta'
	  first=.false.
      endif
     
      z0=inczeta(izmax)*ran2(idum)
      z1=ran2(idum)
      z2=ran2(idum)
      z3=ran2(idum)
      z4=ran2(idum)
      
      i=1
      do while(z0 .gt. inczeta(i))
          i=i+1
      end do

      bbfreq=-log(z1*z2*z3*z4)*Teff/i
      
      return
      end

      subroutine zetainit()
      
      implicit none

      integer i,j
      real temp
      
      integer izmax
      parameter(izmax=145)
      real inczeta
      dimension inczeta(izmax)
      common /izeta/ inczeta

      do i=1,izmax
         inczeta(i)=0.
      end do

      do j=izmax,1,-1
        temp=1./float(j*j*j*j)
        do i=izmax,j,-1
           inczeta(i)=inczeta(i)+temp
        end do
      end do
      
      return
      end
            
 
