      subroutine dustmat(p1,p2,p3,p4,cost,cost2,pi,idust2)

c     a.d. code october 25, 1989
c     revised baw apr 10, 1990
c	**********************************************************************
c
c     this program calculates the elements of the phase matrix for a
c     simple representation of the mrn dust mixture using the algorithms
c     for the ultraviolet region due to richard l. white ap.j. 229, 954,
c     1979.
c     
c     ***********************************************************************
c     
c     cost = cos(angle) of scattering (i.e. angle between incident 
c     photon and scattered photon)
c     g = mean value of cosine of scattering angle (henyey-greenstein)
c     pl = peak linear polarization
c     pc = peak value of linear to circular conversion
c     sc = asymmetry of the circular polarization.
c     p1 = intensity phase function
c     p2 = polarization function
c     p3 = skew polarization
c     p4 = circular polarization
c     
c     the scattering matrix for (i,q,u,v) is of the form
c     
c     p1    p2    0	  0
c     p2    p1    0	  0
c     0     0     p3  -p4
c     0     0     p4   p3
c     
c     **********************************************************************
      implicit none
      include 'dust.com'

      real*8 pi,cost2,cost,p4,p3,p2,p1
      real temp
      integer idust2
c     data g, pl, pc, sc / 0.56, 0.51, 0.39, 1.0 /
c
      temp=g2p1(idust2)-twog(idust2)*cost
c     H-G function
c      p1=onemg2(idust2)/(temp*sqrt(temp))
c     modified Cornette & Shanks function---make sure peak is known
      p1=1.5*onemg2(idust2)/(2.+g2(idust2))*(1.+cost2)/(temp*sqrt(temp))

      p2 = -pl(idust2)*p1*(1.0-cost2)/(1.0+cost2)
      p3 = p1*2.0*cost/(1.0+cost2)
      if(dabs(cost).gt.1.d0) then 
         write(6,*) 'in dustmat, cost.gt.1',cost
         if(cost.gt.1.d0) then     
            cost=1.d0
         else    
            cost=-1. d0    
         end if 
      end if
c     circular polarization, comment out if you don't care
c     angle in degrees!
c     phi=acosd(cost)
c     f=3.13*phi*exp(-7.0*phi/180.)
c     now convert to radians
c     f2=(phi+sc*f)*pi/180.
c     fphi= (1+3.13*sc*exp(-7.0*phi/pi))*phi
c     c=(cos(f2))**2
c     p4 = -pc*p1*(1-c)/(1+c)
      p4=0.d0
      
      return
      end
