c **********************************************************************
      subroutine atmosinit(atmosfile)
c ----------------------------------------------------------------------
c   Reads in a two-column array containing wavelengths and fluxes
c   Calculates partial integrals at every wavelength point
c   Normalizes the integrals on the bolometric flux
c   Uses real function tr2 for integration
c-----------------------------------------------------------------------
c                                [Anatoly Miroshnichenko, Mar. 1998]
c
c     b.a.whitney July 2002, bug in sampling (atmosfreq), can't find it, 
c     rewriting to something I can understand:
c     reorder frequency array, linearly interpolate cdf
c ======================================================================       
      
      implicit none
      
      include 'dust.com'
      
      integer nHnu,nHnumax
      parameter(nHnumax=60000)
      real*8 atmosnu,Hnuint,lognu,logHnu
      dimension atmosnu(nHnumax),Hnuint(nHnumax),lognu(nHnumax)
     +     ,logHnu(nHnumax)
      common /atmos_interp/atmosnu,Hnuint,lognu,logHnu,nHnu

      CHARACTER*80 atmosfile
      CHARACTER*80 header1,header2,header3
      INTEGER i,n,id
      REAL*8 winv(nHnumax),ftmp(nHnumax),f(nHnumax),chif(nHnumax)
     +     ,winvtmp(nHnumax),fbol,tr2,wtmp
c-----------------------------------------------------------------------

      open(21,file=atmosfile,status='old')
      read(21,*) header1
c      read(21,*) header2
c      read(21,*) header3
      
      do i=1,nHnumax
c	old kurucz format 
c         read(21,*,end=100) wtmp,winvtmp(i),ftmp(i)
c	tom's new format
         read(21,*,end=100) wtmp,ftmp(i)
         winvtmp(i)=1./wtmp
      enddo
      
      write(*,*) 'atmosinit: too many frequencies in model atmosphere'
      stop

 100  close(21)

	do id=1,4
	
      nHnu=i-1   
      write(*,*) 'nHnu=',nHnu

c     reorder arrays with increasing frequency
       do i=1,nHnu
          winv(nHnu-i+1)=winvtmp(i)
          f(nHnu-i+1)=ftmp(i)
       enddo

c     integrate pdf
       do i=1,nHnu
          atmosnu(i)=1.2398*winv(i)
          lognu(i)=dlog10(atmosnu(i))
          call opacset(sngl(atmosnu(i)))
          chif(i)=kappa(id)*f(i)

          if(i.eq.1) then
             Hnuint(i)=0.
          else
             Hnuint(i)=Hnuint(i-1)+0.5*(f(i)+f(i-1))*
     +            (winv(i)-winv(i-1))
          endif
c          print*,winv(i),atmosnu(i),chif(i)
      end do
      
      fbol=Hnuint(nHnu)

      if (fbol.eq.0) then
         print*,''
         print*,'***************'
         print*,'ERROR, atmosphere file is all ZEROs. use another.'
         print*,'This is a problem with the downloaded kurucz'
         print*,'models.   program stopping'
         stop
      endif

      kappaf(id)=0.
      do i=1,nHnu
            Hnuint(i)=Hnuint(i)/fbol
            if (i.gt.1) logHnu(i)=dlog10(Hnuint(i))
            if (i.eq.1) then
               kappaf(id)=0.
            else
               kappaf(id)=kappaf(id)+0.5*(chif(i)+chif(i-1))*
     +              (winv(i)-winv(i-1))
            endif
c            print*,winv(i),chif(i),kappaf
      end do
      kappaf(id)=kappaf(id)/fbol

      fbol=fbol*2.997925e14
      write(*,*) 'Teff,kappaf=',(4.*3.141593*fbol/5.67051e-5)**0.25,
     +           kappaf(id)

	enddo
		
      return
      end
      

      real function atmosfreq()

c     baw, July 2002

      implicit none

      include 'random.com'
      
      integer i
      real xi

      integer nHnu,nHnumax
      parameter(nHnumax=60000)
      real*8 atmosnu,Hnuint,lognu,logHnu,logatm
      dimension atmosnu(nHnumax),Hnuint(nHnumax),lognu(nHnumax)
     +     ,logHnu(nHnumax)
      common /atmos_interp/atmosnu,Hnuint,lognu,logHnu,nHnu

      real ran2
      
      xi=ran2(idum)

      call locate2(Hnuint,nHnumax,nHnu,dble(xi),i)

c     function is linear in log so do interpolation in logspace.
      if (i.gt.1) then
         logatm=lognu(i)+(lognu(i+1)-lognu(i))/(logHnu(i+1)-logHnu(i))
     +        *(alog10(xi)-logHnu(i))
         atmosfreq=10**logatm
      else
c         Hnuint(1)=0. so log won't work
      atmosfreq=atmosnu(i)+(atmosnu(i+1)-atmosnu(i))/
     +     (Hnuint(i+1)-Hnuint(i))*(xi-Hnuint(i))
      endif

      return
      end
